"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvaluationRecord = exports.Evaluation = void 0;
const css_select_1 = __importDefault(require("css-select"));
const crypto_1 = require("crypto");
const util_1 = require("@qualweb/util");
const evaluationRecord_object_1 = __importDefault(require("./evaluationRecord.object"));
exports.EvaluationRecord = evaluationRecord_object_1.default;
const wappalyzer_1 = require("@qualweb/wappalyzer");
class Evaluation {
    getEvaluator(page, sourceHtml, url) {
        return __awaiter(this, void 0, void 0, function* () {
            const [plainHtml, pageTitle, elements, browserUserAgent,] = yield Promise.all([
                page.evaluate(() => {
                    return document.documentElement.outerHTML;
                }),
                page.title(),
                page.$$("*"),
                page.browser().userAgent(),
            ]);
            let urlStructure = undefined;
            if (url) {
                urlStructure = this.parseUrl(url, page.url() !== "about:blank" ? page.url() : url);
            }
            const processedHtml = {
                html: {
                    plain: plainHtml,
                },
                title: pageTitle,
                elementCount: elements.length,
            };
            const viewport = page.viewport();
            const evaluator = {
                name: "QualWeb",
                description: "QualWeb is an automatic accessibility evaluator for webpages.",
                version: "3.0.0",
                homepage: "http://www.qualweb.di.fc.ul.pt/",
                date: new Date().toISOString().replace(/T/, " ").replace(/\..+/, ""),
                hash: crypto_1.randomBytes(40).toString("hex"),
                url: urlStructure,
                page: {
                    viewport: {
                        mobile: !!viewport.isMobile,
                        landscape: !!viewport.isLandscape,
                        userAgent: browserUserAgent,
                        resolution: {
                            width: viewport.width,
                            height: viewport.height,
                        },
                    },
                    dom: {
                        source: sourceHtml,
                        processed: processedHtml,
                    },
                },
            };
            return evaluator;
        });
    }
    addQWPage(page) {
        return __awaiter(this, void 0, void 0, function* () {
            yield page.addScriptTag({
                path: require.resolve("@qualweb/qw-page"),
            });
            yield page.evaluate(() => {
                window.page = new QWPage.QWPage(document, window, true);
            });
            yield page.keyboard.press("Tab");
            yield page.evaluate(() => {
                window.page72 = new QWPage.QWPage(document, window, true);
            });
        });
    }
    executeACT(page, sourceHtml, options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield page.addScriptTag({
                path: require.resolve("@qualweb/act-rules"),
            });
            const metaElements = css_select_1.default("meta", sourceHtml.html.parsed);
            const parsedMetaElements = new Array();
            for (const element of metaElements || []) {
                if (!!element) {
                    const content = util_1.DomUtils.getSourceElementAttribute(element, "content");
                    const httpEquiv = util_1.DomUtils.getSourceElementAttribute(element, "http-equiv");
                    const htmlCode = util_1.DomUtils.getSourceElementHtmlCode(element, true, false);
                    const selector = util_1.DomUtils.getSourceElementSelector(element);
                    parsedMetaElements.push({
                        content,
                        httpEquiv,
                        htmlCode,
                        selector,
                    });
                }
            }
            const actReport = yield page.evaluate((parsedMetaElements, options) => {
                const act = new ACTRules.ACTRules(options);
                return act.execute(parsedMetaElements, window.page);
            }, parsedMetaElements, options);
            const r40 = "QW-ACT-R40";
            const r72 = "QW-ACT-R72";
            if (!options ||
                !options["rules"] ||
                options["rules"].includes(r40) ||
                options["rules"].includes("59br37")) {
                const viewport = page.viewport();
                yield page.setViewport({
                    width: 640,
                    height: 512,
                });
                const actReportR40 = yield page.evaluate(() => {
                    const act = new ACTRules.ACTRules();
                    return act.executeQW_ACT_R40(window.page);
                });
                yield page.setViewport({
                    width: viewport.width,
                    height: viewport.height,
                });
                actReport.assertions[r40] = actReportR40;
                let outcome = actReportR40.metadata.outcome;
                if (outcome === "passed") {
                    actReport.metadata.passed++;
                }
                else if (outcome === "failed") {
                    actReport.metadata.failed++;
                }
                else if (outcome === "warning") {
                    actReport.metadata.warning++;
                }
                else {
                    actReport.metadata.inapplicable++;
                }
            }
            if (!options ||
                !options["rules"] ||
                options["rules"].includes(r72) ||
                options["rules"].includes("8a213c")) {
                const actReportR72 = yield page.evaluate(() => {
                    const act = new ACTRules.ACTRules();
                    return act.executeQW_ACT_R72(window.page72);
                });
                actReport.assertions[r72] = actReportR72;
                let outcome = actReportR72.metadata.outcome;
                if (outcome === "passed") {
                    actReport.metadata.passed++;
                }
                else if (outcome === "failed") {
                    actReport.metadata.failed++;
                }
                else if (outcome === "warning") {
                    actReport.metadata.warning++;
                }
                else {
                    actReport.metadata.inapplicable++;
                }
            }
            return actReport;
        });
    }
    executeWCAG(page, options, validation) {
        return __awaiter(this, void 0, void 0, function* () {
            yield page.addScriptTag({
                path: require.resolve("@qualweb/wcag-techniques"),
            });
            const url = page.url();
            const newTabWasOpen = yield util_1.BrowserUtils.detectIfUnwantedTabWasOpened(page.browser(), url);
            const htmlReport = yield page.evaluate((newTabWasOpen, validation, options) => {
                const html = new WCAGTechniques.WCAGTechniques(options);
                return html.execute(window.page, newTabWasOpen, validation);
            }, newTabWasOpen, validation, options);
            return htmlReport;
        });
    }
    executeBP(page, options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield page.addScriptTag({
                path: require.resolve("@qualweb/best-practices"),
            });
            const bpReport = yield page.evaluate((options) => {
                const bp = new BestPractices.BestPractices();
                if (options)
                    bp.configure(options);
                return bp.execute(window.page);
            }, options);
            return bpReport;
        });
    }
    executeCounter(page) {
        return __awaiter(this, void 0, void 0, function* () {
            yield page.addScriptTag({
                path: require.resolve("@qualweb/counter"),
            });
            const Counter = yield page.evaluate(() => {
                return Counter.executeCounter(window.page);
            });
            return Counter;
        });
    }
    evaluatePage(sourceHtml, page, execute, options, url, validation) {
        return __awaiter(this, void 0, void 0, function* () {
            const evaluator = yield this.getEvaluator(page, sourceHtml, url);
            const evaluation = new evaluationRecord_object_1.default(evaluator);
            yield this.addQWPage(page);
            if (execute.act) {
                evaluation.addModuleEvaluation("act-rules", yield this.executeACT(page, sourceHtml, options["act-rules"]));
            }
            if (execute.wcag) {
                evaluation.addModuleEvaluation("wcag-techniques", yield this.executeWCAG(page, options["wcag-techniques"], validation));
            }
            if (execute.bp) {
                evaluation.addModuleEvaluation("best-practices", yield this.executeBP(page, options["best-practices"]));
            }
            if (execute.wappalyzer) {
                evaluation.addModuleEvaluation("wappalyzer", yield wappalyzer_1.executeWappalyzer(url));
            }
            if (execute.counter) {
                evaluation.addModuleEvaluation("counter", yield this.executeCounter(page));
            }
            return evaluation;
        });
    }
    parseUrl(url, pageUrl) {
        const inputUrl = url;
        let protocol;
        let domainName;
        let domain;
        let uri;
        let completeUrl = pageUrl;
        protocol = completeUrl.split("://")[0];
        domainName = completeUrl.split("/")[2];
        const tmp = domainName.split(".");
        domain = tmp[tmp.length - 1];
        uri = completeUrl.split("." + domain)[1];
        const parsedUrl = {
            inputUrl,
            protocol,
            domainName,
            domain,
            uri,
            completeUrl,
        };
        return parsedUrl;
    }
}
exports.Evaluation = Evaluation;
//# sourceMappingURL=index.js.map